@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -25,15 +24,17 @@
 #include <string.h>
 
 #include <glib.h>
+#include <glib/gi18n-lib.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdk.h>
+#if defined(GDK_WINDOWING_WAYLAND)
+#include <gdk/gdkwayland.h>
+#endif
 
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include "gnome-idle-monitor.h"
 #include "meta-dbus-idle-monitor.h"
 
-#define GNOME_IDLE_MONITOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNOME_TYPE_IDLE_MONITOR, GnomeIdleMonitorPrivate))
-
 G_STATIC_ASSERT(sizeof(unsigned long) == sizeof(gpointer));
 
 struct _GnomeIdleMonitorPrivate
@@ -78,6 +79,7 @@ static void add_idle_watch (GnomeIdleMonitor *, GnomeIdleMonitorWatch *);
 static void add_active_watch (GnomeIdleMonitor *, GnomeIdleMonitorWatch *);
 
 G_DEFINE_TYPE_WITH_CODE (GnomeIdleMonitor, gnome_idle_monitor, G_TYPE_OBJECT,
+			 G_ADD_PRIVATE (GnomeIdleMonitor)
 			 G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE,
 						gnome_idle_monitor_initable_iface_init))
 
@@ -391,14 +393,12 @@ gnome_idle_monitor_class_init (GnomeIdleMonitorClass *klass)
 				     GDK_TYPE_DEVICE,
 				     G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
 	g_object_class_install_property (object_class, PROP_DEVICE, obj_props[PROP_DEVICE]);
-
-	g_type_class_add_private (klass, sizeof (GnomeIdleMonitorPrivate));
 }
 
 static void
 gnome_idle_monitor_init (GnomeIdleMonitor *monitor)
 {
-	monitor->priv = GNOME_IDLE_MONITOR_GET_PRIVATE (monitor);
+	monitor->priv = gnome_idle_monitor_get_instance_private (monitor);
 
 	monitor->priv->watches = g_hash_table_new_full (NULL,
 							NULL,
@@ -430,12 +430,21 @@ gnome_idle_monitor_new (void)
  * Returns: a new #GnomeIdleMonitor that tracks the device-specific
  * idletime for @device. If device-specific idletime is not available,
  * %NULL is returned, and @error is set. To track server-global
- * idletime for all devices, use gnome_idle_monitor_new().
+ * idletime for all devices, use gnome_idle_monitor_new(). This function
+ * only works under X11, not under Wayland and will return an error.
  */
 GnomeIdleMonitor *
 gnome_idle_monitor_new_for_device (GdkDevice  *device,
 				   GError    **error)
 {
+#if defined(GDK_WINDOWING_WAYLAND)
+	if (GDK_IS_WAYLAND_DISPLAY (gdk_display_get_default ())) {
+		g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+				     "Per-device idle monitors not supported under Wayland");
+		return NULL;
+	}
+#endif
+
 	return GNOME_IDLE_MONITOR (g_initable_new (GNOME_TYPE_IDLE_MONITOR, NULL, error,
 						   "device", device, NULL));
 }
@@ -522,7 +531,8 @@ add_active_watch (GnomeIdleMonitor      *monitor,
 /**
  * gnome_idle_monitor_add_idle_watch:
  * @monitor: A #GnomeIdleMonitor
- * @interval_msec: The idletime interval, in milliseconds
+ * @interval_msec: The idletime interval, in milliseconds. It must be
+ *     a strictly positive value (> 0).
  * @callback: (allow-none): The callback to call when the user has
  *     accumulated @interval_msec milliseconds of idle time.
  * @user_data: (allow-none): The user data to pass to the callback
@@ -551,6 +561,7 @@ gnome_idle_monitor_add_idle_watch (GnomeIdleMonitor	       *monitor,
 	GnomeIdleMonitorWatch *watch;
 
 	g_return_val_if_fail (GNOME_IS_IDLE_MONITOR (monitor), 0);
+	g_return_val_if_fail (interval_msec > 0, 0);
 
 	watch = make_watch (monitor,
 			    interval_msec,
